import RPi.GPIO as gpio
import time

gpio.setmode(gpio.BCM)

print "RPI Board Revision: " + str(gpio.RPI_REVISION)

VFD_OE = 7
VFD_D7 = 4
VFD_D6 = 14
VFD_D5 = 15
VFD_D4 = 17
VFD_D3 = 18
VFD_D2 = 27
VFD_D1 = 22
VFD_D0 = 23
VFD_WR = 24
VFD_CD = 10
VFD_RD = 9
VFD_CS = 25
VFD_FRP = 11
VFD_BL = 8
VFD_OE = 7

# This VFD has 15Bit Address Structure (0x00 ~ 0x1ff)
# Lower(8Bit) and Upper (5Bit)

def lobyte(x):
    return ( ((x) & 0x00ff) )
def hibyte(x):
    return ( (((x) & 0x1f00) >> 8) )

            # VFD Controller Board Configurations
VFD_SRAM_SZ =          8192              # embedded display memory size (8Kbyte:8192 x 8 bits)

            # Cursor address behavior
VFD_ADDR_AUTO_INC =    0                 # Cursor address Increment behavior 0:Hold,1:Autoincrement)

            # Display properties
            # Graphic Area, Screen #1
VFD_X_PIXEL  =        256               # The number of Width  Dots
VFD_Y_PIXEL  =        64                # The number of Height Dots
VFD_Y_PAGES  =        8                 # The number of Page (8page x 8bit = 64 pixels)
VFD_X_CENTER =        (VFD_X_PIXEL / 2)                # the column  center of pixel
VFD_Y_CENTER =        (VFD_Y_PIXEL / 2)                # the row     center of pixel

            # Text Area, Screen #2(Power On Default)
VFD_TEXT_FONT_WIDTH  = 6                 # 6:6x8
VFD_TEXT_FONT_HEIGHT = 8                 # 8:6x8
VFD_X_TEXT         =  (VFD_X_PIXEL / VFD_TEXT_FONT_WIDTH)               # The number of Char per Column
VFD_Y_TEXT         =  (VFD_X_PIXEL / VFD_TEXT_FONT_HEIGHT)              # The number of Char per Line
VFD_X_TEXT_CENTER  = ((VFD_X_PIXEL / 8) / 2)              # the column center of text
VFD_Y_TEXT_CENTER  =  (VFD_Y_PAGES / 2)                   # the row    center of Text
VFD_TEXT_VTA       =   3                 # Virtual Screen Adddress (VFD_X_TEXT * VFD_TEXT_VTA)
                                                # Refer. Datasheet 12.2 Character Display map
            # indicate how many bytes are used for one complete row (number of Column)
VFD_GRAPHIC_AREA   =  (VFD_X_PIXEL)
VFD_TEXT_AREA      =  (VFD_X_TEXT)

            # indicate how many bytes are used for Graphic area and Text area
VFD_GRAPHIC_SIZE   =  (VFD_GRAPHIC_AREA * VFD_Y_PAGES)
VFD_TEXT_SIZE      =  (VFD_TEXT_AREA    * VFD_TEXT_FONT_HEIGHT)

            # They are used to set the beginning of the text area and graphic area in the RAM
VFD_GRAPHIC_HOME   =   0
VFD_TEXT_HOME      =  (VFD_GRAPHIC_HOME + VFD_GRAPHIC_SIZE)

            # command function equates for Noritake GU256X64-901A VFD Controller
            # * Marks are Power On Default
VFD_SCREEN_OFF     =   0x00               # *Screen #1,#2 Off
VFD_SCREEN1_ON     =   0x01               #  Screen #1 On, #2 Off (Graphic)
VFD_SCREEN2_ON     =   0x02               #  Screen #2 On, #1 Off (Text, Power On Default)
VFD_SCREEN_ON      =   0x03               #  Screen #1,#2 On
VFD_ADDR_INC_AUTO  =   0x04               #  Automatic Increment of Cursor address
VFD_ADDR_INC_HOLD  =   0x05               #  Hold Increment of Cursor address
VFD_SCREEN_OR      =   0x10               # *OR  Display Screen #1,#2
VFD_SCREEN_XOR     =   0x11               #  XOR Display Screen #1,#2
VFD_SCREEN_AND     =   0x12               #  AND Display Screen #1,#2
VFD_DISPLAY_ON     =   0x14               # *Display On
VFD_DISPLAY_OFF    =   0x15               #  Display Off
VFD_CURSOR_INC_ON  =   0x04               # *Cursor is automatically increment
VFD_CURSOR_INC_OFF =   0x05               #  Cursor is Not automatically increment
VFD_SCREEN2_CHAR   =   0x06               # *Screen #2 is selected to Character Display
VFD_SCREEN2_GRAP   =   0x07               #  Screen #2 is selected to Graphic Display
VFD_DATA_WRITE     =   0x08               # Set Data(pattern or character) Write mode
VFD_DATA_READ      =   0x09               # Set Data(pattern or character) Read mode
VFD_START_ADDR_L1  =   0x0a               # Lower byte of Screen#1(Graphic) address
VFD_START_ADDR_U1  =   0x0b               # Upper byte of Screen#1(Graphic) address
VFD_START_ADDR_L2  =   0x0c               # Lower byte of Screen#2(Text) address
VFD_START_ADDR_U2  =   0x0d               # Upper byte of Screen#2(Text) address

            # The CURSOR address means writing/reading address of displaying data in the RAM
VFD_CURSOR_ADDR_L  =   0x0e               # Set Lower byte of cursor address
VFD_CURSOR_ADDR_U  =   0x0f               # Set Upper byte of cursor address

VFD_BRIGHTNESS_100 =   0x18               # *Set brightness 100.0%
VFD_BRIGHTNESS_87  =   0x19               #  Set brightness 87.5%
VFD_BRIGHTNESS_75  =   0x1a               #  Set brightness 75.0%
VFD_BRIGHTNESS_62  =   0x1b               #  Set brightness 62.5%
VFD_BRIGHTNESS_42  =   0x1d               #  Set brightness 62.5%

# VFD Color modes
VFD_DOT_CLR        =   0
VFD_DOT_SET        =   1
VFD_DOT_XOR        =   2

VFD_LINE_SOLID     = 0
VFD_LINE_DOTTED    = 1

gpio.setup(VFD_D0, gpio.OUT) # Data 0
gpio.setup(VFD_D1, gpio.OUT) # Data 1
gpio.setup(VFD_D2, gpio.OUT) # Data 2
gpio.setup(VFD_D3, gpio.OUT) # Data 3
gpio.setup(VFD_D4, gpio.OUT) # Data 4
gpio.setup(VFD_D5, gpio.OUT) # Data 5
gpio.setup(VFD_D6, gpio.OUT) # Data 6
gpio.setup(VFD_D7, gpio.OUT)  # Data 7
gpio.setup(VFD_WR, gpio.OUT) # Write if low
gpio.setup(VFD_CD, gpio.OUT) # Command if high, Data if low
gpio.setup(VFD_RD, gpio.OUT)  # Read if low
gpio.setup(VFD_CS, gpio.OUT) # Chip select if low
gpio.setup(VFD_FRP, gpio.IN) # Frame buffer read
gpio.setup(VFD_BL, gpio.OUT)  # Blank if low
gpio.setup(VFD_OE, gpio.OUT)  # Output enable if low for level translator

gpio.output(VFD_OE, True) # Enable translators

def clr(pin):
    gpio.output(pin, False)

def set(pin):
    gpio.output(pin, True)

def vfd_write_toggle():
    clr(VFD_WR)
    time.sleep(0.005)
    set(VFD_WR)
    time.sleep(0.001)

def vfd_data(dbyte):
    gpio.output(VFD_D0, dbyte & 0x01)
    gpio.output(VFD_D1, dbyte & 0x02)
    gpio.output(VFD_D2, dbyte & 0x04)
    gpio.output(VFD_D3, dbyte & 0x08)
    gpio.output(VFD_D4, dbyte & 0x10)
    gpio.output(VFD_D5, dbyte & 0x20)
    gpio.output(VFD_D6, dbyte & 0x40)
    gpio.output(VFD_D7, dbyte & 0x80)

def vfd_command(command):
    set(VFD_RD)
    set(VFD_CD)
    vfd_data(command)
    vfd_write_toggle()

def vfd_set_cursor_address(addr):
    vfd_command(VFD_CURSOR_ADDR_L)
    vfd_data(lobyte(addr))
    vfd_command(VFD_CURSOR_ADDR_U)
    vfd_data(hibyte(addr))

def vfd_init():
    time.sleep(1)
    vfd_cx = 0
    vfd_cy = 0

    set(VFD_CS)
    set(VFD_RD)
    set(VFD_WR)
    set(VFD_CD)

    clr(VFD_CS)

    vfd_set_cursor_address(0)
    vfd_command(VFD_DATA_WRITE)

    for i in range (0, VFD_SRAM_SZ):
	vfd_data(0x00)

    # Screen#1 (Graphic) Home Address
    vfd_command(VFD_START_ADDR_L1)
    vfd_data(lobyte(VFD_GRAPHIC_HOME))
    vfd_command(VFD_START_ADDR_U1)
    vfd_data(hibyte(VFD_GRAPHIC_HOME))

    # Screen#2 (Text) Home Address
    vfd_command(VFD_START_ADDR_L2)
    vfd_data(lobyte(VFD_TEXT_HOME))
    vfd_command(VFD_START_ADDR_U2)
    vfd_data(hibyte(VFD_TEXT_HOME))

    if(VFD_ADDR_AUTO_INC):
	vfd_command(VFD_ADDR_INC_AUTO)   # use Automatic Increment of Cursor address
    else:
	vfd_command(VFD_ADDR_INC_HOLD)   # do not use Automatic Increment of Cursor address

    vfd_command(VFD_BRIGHTNESS_42)  # omit this command, because 100%  is Power On. Defualt

    vfd_command(VFD_SCREEN_ON);     # Critical item(At Power on, Screen#1,#2 are Set to OFF)

vfd_init()